#ifndef FrameCPP_VERSION_8_FrSimData_HH
#define FrameCPP_VERSION_8_FrSimData_HH

#include <vector>

#include "framecpp/Common/SearchContainer.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version7/FrSimData.hh"

#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/STRING.hh"

#include "framecpp/Version8/FrHistory.hh"
#include "framecpp/Version8/FrTable.hh"
#include "framecpp/Version8/FrVect.hh"

namespace FrameCPP
{
  namespace Version_8
  {
    //===================================================================
    //===================================================================

    typedef Previous::FrSimData::FrSimDataStorage FrSimDataStorage;

    //===================================================================
    // FrSimDataRefs
    //===================================================================

    using Previous::FrSimDataRefsT;

    typedef FrSimDataRefsT< FrVect, FrTable> FrSimDataRefs;

    //===================================================================
    //===================================================================
    class FrSimData
      : public ObjectWithChecksum< Common::FrameSpec::Object >,
	public FrSimDataStorage,
	public FrSimDataRefs,
	public Common::TOCInfo
    {
    public:
      //: Default constructor
      FrSimData( );
      FrSimData( const FrSimData& SimData );
      FrSimData( const std::string& Name,
		 const std::string& Comment,
		 const REAL_8 SampleRate, 
		 const REAL_8 FShift,
		 const REAL_4 Phase,
		 const REAL_8 TimeOffset = DEFAULT_TIME_OFFSET );
      FrSimData( Previous::FrSimData& Source,
		 Common::IStream* Stream );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      virtual const char* const ObjectStructName( ) const;

      /// \brief Retrive the name
      const std::string& GetNameSlow() const;
      
      //: Merge with another FrAdcData
      FrSimData& Merge( const FrSimData& RHS );

      /// \brief comparison operator
      bool operator==( const FrSimData& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      virtual void TOCQuery( int InfoClass, ... ) const;

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Upgrade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      //-----------------------------------------------------------------
      cmn_streamsize_type pBytes( const Common::StreamBase& Stream) const;

      virtual FrSimData* pCreate( Common::IStream& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      //-----------------------------------------------------------------
      virtual void pWrite( Common::OStream& Stream ) const;

    private:
      //-----------------------------------------------------------------
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrameSpec::Object > object_type;

      using Common::FrameSpec::Object::Create;

      FrSimData( Common::IStream& Stream );
    };

    inline std::auto_ptr< Common::FrameSpec::Object > FrSimData::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrSimData, FrSimData >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    inline const char* const FrSimData::
    StructName( )
    {
      static const char* const name = "FrSimData";
      return name;
    }

    inline bool FrSimData::
    operator==( const FrSimData& RHS ) const
    {
      return ( ( this == &RHS )
	       || ( ( m_data == RHS.m_data )
		    && ( m_refs == RHS.m_refs )
		    ) );
    }
  } // namespace - Version_8
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_8_FrSimData_HH */
