#ifndef FrameCPP_VERSION_8_FrTable_HH
#define FrameCPP_VERSION_8_FrTable_HH

#include <vector>

#include "framecpp/Version8/FrameSpec.hh"

#include "framecpp/Common/SearchContainer.hh"

#include "framecpp/Version7/FrTable.hh"

#include "framecpp/Version8/PTR_STRUCT.hh"
#include "framecpp/Version8/FrVect.hh"
#include "framecpp/Version8/STRING.hh"

namespace FrameCPP
{
  namespace Version_8
  {
    //===================================================================
    //
    //===================================================================
    typedef Previous::FrTable::FrTableNPS FrTableNPS;

    //===================================================================
    //
    //===================================================================
    typedef Version_4::
    FrTablePST< PTR_STRUCT,
		Common::SearchContainer< FrVect, &FrVect::GetName > >
    FrTablePS;

    //===================================================================
    //
    //===================================================================
    class FrTable
      : public ObjectWithChecksum< Common::FrameSpec::Object >,
	public FrTableNPS,
	public FrTablePS
    {
    public:
      //: Default constructor
      FrTable( );

      FrTable( const FrTable& Source );

      FrTable( const FrTableNPS& Source );

      FrTable( const std::string& name, INT_4U nrows );

      FrTable( Previous::FrTable& Source, Common::IStream* Stream );

      virtual ~FrTable( );

      virtual const char* const ObjectStructName( ) const;

      const std::string& GetNameSlow( ) const;

      //: Merge with another FrAdcData
      FrTable& Merge( const FrTable& RHS );

      //: Concatination operator
      FrTable& operator+=( const FrTable& RHS );

      static std::auto_ptr< Object >
      Promote( INT_2U Source,
	       Object* Obj, Common::IStream* Stream );

      //-----------------------------------------------------------------
      /// \brief Dictionary class id for this structure.
      //-----------------------------------------------------------------
      static const Common::FrameSpec::Info::frame_object_types CLASS_ID
      = Common::FrameSpec::Info::FSI_FR_TABLE;

      //-----------------------------------------------------------------
      /// \brief Description of the structure
      //-----------------------------------------------------------------
      static const Common::Description* StructDescription( );

      //-----------------------------------------------------------------
      /// \brief Name of the stucture
      //-----------------------------------------------------------------
      static const char* const StructName( );

      /// \brief comparison operator
      bool operator==( const FrTable& RHS ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

    protected:
      /// \brief Down grade an object
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      /// \brief Upgrade an object
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

      virtual cmn_streamsize_type
      pBytes( const Common::StreamBase& Stream ) const;

      virtual FrTable* pCreate( Common::IStream& Stream ) const;

      virtual void pWrite( Common::OStream& Stream ) const;

    private:
      //-----------------------------------------------------------------
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrameSpec::Object > object_type;

      using Common::FrameSpec::Object::Create;

      FrTable( Common::IStream& Stream );
    };

    //-------------------------------------------------------------------
    /// Tests the equality of two tables.
    ///
    /// \return
    ///     True if the two tables are equivelant, false otherwise.
    //-------------------------------------------------------------------
    inline bool FrTable::
    operator==( const FrTable& RHS ) const
    {
      return ( m_data == RHS.m_data );
    }

    inline std::auto_ptr< Common::FrameSpec::Object > FrTable::
    Promote( INT_2U Source,
	     Common::FrameSpec::Object* Obj, Common::IStream* Stream )
    {
      return Object::PromoteObject< Previous::FrTable, FrTable >
	( DATA_FORMAT_VERSION, Source, Obj, Stream );
    }

    //-------------------------------------------------------------------
    /// The name of the structure is most often used when writing the
    /// description of the structure out to a stream.
    ///
    /// \return
    ///     The name of the structure.
    //-------------------------------------------------------------------
    inline const char* const FrTable::
    StructName( )
    {
      static const CHAR* class_name( "FrTable" );
      return class_name;
    }

  } // namespace - Version_8
} // namespace FrameCPP
#endif /* FrameCPP_VERSION_8_FrTable_HH */

