#ifndef GENERAL__REVERSE_HH
#define GENERAL__REVERSE_HH

#include "general/config.h"

#include <assert.h>

#if HAVE_BYTESWAP_H
#include "byteswap.h"
#elif HAVE_SYS_BYTEORDER_H
#include "sys/types.h"
#include "sys/byteorder.h"

#ifndef bswap_16
#define bswap_16(x) BSWAP_16(x)
#endif /* bswap_16 */
#ifndef bswap_32
#define bswap_32(x) BSWAP_32(x)
#endif /* bswap_32 */
#ifndef bswap_64
#define bswap_64(x) BSWAP_64(x)
#endif /* bswap_64 */
#else
#define BSWAP_8(x)      ((x) & 0xff)
#define bswap_16(x)     ((BSWAP_8(x) << 8) | BSWAP_8((x) >> 8))
#define bswap_32(x)     ((bswap_16(x) << 16) | bswap_16((x) >> 16))
#define bswap_64(x)     ((bswap_32(x) << 32) | bswap_32((x) >> 32))
#endif /* HAVE_SYS_BYTEORDER_H */

#include "general/types.hh"

//-----------------------------------------------------------------------------
/// \brief  Reverse Byte Order
///
/// \param s
/// \param n
///
template< size_t SIZE > void reverse( void* s, size_t n );

//-----------------------------------------------------------------------------
/// \brief  Reverse 1 byte
///
/// This does nothing.
///
/// \param dest
/// \param n
///
template<>
inline void
reverse< 1 >( void* dest, size_t n )
{
    return;
}


//-----------------------------------------------------------------------------
/// \brief  Reverse 2 bytes
///
/// Reverses the order of two bytes.  This is a template specialization.
///
/// \param dest
/// \param n
///
template<>
inline void
reverse< 2 >( void* dest, size_t n )
{
    register INT_2U* d = reinterpret_cast< INT_2U* >( dest );
    register INT_2U* end = d + n;

    while ( d != end )
    {
        *d = bswap_16( *d );
        ++d;
    }
}


//-----------------------------------------------------------------------------
/// \brief  Reverse 4 bytes
///
/// Reverses the order of four bytes.  This is a template specialization.
///
/// \param dest
/// \param n
///
template<>
inline void
reverse< 4 >( void* dest, size_t n )
{
    register INT_4U* d = reinterpret_cast< INT_4U* >( dest );
    register INT_4U* end = d + n;

    while ( d != end )
    {
        *d = bswap_32( *d );
        ++d;
    }
}


//-----------------------------------------------------------------------------
/// \brief  Reverse 8 bytes
///
/// Reverses the order of eight bytes.  This is a template specialization.
///
/// \param dest
/// \param n
///
template<>
inline void
reverse< 8 >( void* dest, size_t n )
{
    register INT_8U* cur = reinterpret_cast< INT_8U* >( dest );
    register INT_8U* end = cur + n;

    while ( cur != end )
    {
        *cur = bswap_64( *cur );
	++cur;
    }
}

//-----------------------------------------------------------------------------
/// \brief  Reverse 36 bytes
///
/// Reverses the order of eight bytes.  This is a template specialization.
///
/// \param dest
/// \param n
///
template<>
inline void
reverse< 12U >( void* s, size_t n )
{
  assert( 0 );
}

//-----------------------------------------------------------------------------
/// \brief  Reverse 36 bytes
///
/// Reverses the order of eight bytes.  This is a template specialization.
///
/// \param dest
/// \param n
///
template<>
inline void
reverse< 36U >( void* s, size_t n )
{
  assert( 0 );
}

#endif /* GENERAL__REVERSE_HH */
